@@ -30,6 +30,8 @@
 
 GMainLoop *main_loop;
 
+static GCancellable *cancellable;
+
 static gboolean async = FALSE;
 static GOptionEntry entries[] =
 {
@@ -41,7 +43,13 @@ static GOptionEntry entries[] =
 static void
 interrupt_signal_handler (G_GNUC_UNUSED int signum)
 {
-        g_main_loop_quit (main_loop);
+        if (!async || g_cancellable_is_cancelled (cancellable)) {
+                g_main_loop_quit (main_loop);
+        } else {
+                g_print ("Canceling all introspection calls. "
+                         "Press ^C again to quit.\n");
+                g_cancellable_cancel (cancellable);
+        }
 }
 
 static void
@@ -202,9 +210,10 @@ service_proxy_available_cb (G_GNUC_UNUSED GUPnPControlPoint *cp,
         info = GUPNP_SERVICE_INFO (proxy);
 
         if (async) {
-                gupnp_service_info_get_introspection_async (info,
-                                                            got_introspection,
-                                                            NULL);
+                gupnp_service_info_get_introspection_async_full (info,
+                                                                 got_introspection,
+                                                                 cancellable,
+                                                                 NULL);
         } else {
                 introspection =
                         gupnp_service_info_get_introspection (info, &error);
@@ -256,9 +265,6 @@ main (int argc, char **argv)
                 return EXIT_FAILURE;
         }
 		
-#if !GLIB_CHECK_VERSION(2,35,0)
-        g_type_init ();
-#endif
 
         error = NULL;
         context = g_initable_new (GUPNP_TYPE_CONTEXT, NULL, &error, NULL);
@@ -270,6 +276,8 @@ main (int argc, char **argv)
                 return EXIT_FAILURE;
         }
 
+        cancellable = g_cancellable_new ();
+
         /* We're interested in everything */
         cp = gupnp_control_point_new (context, "ssdp:all");
 
